@@ -107,44 +107,44 @@ static PyObject *((*tailOf)(xmlNode (*)));
 static PyObject *((*textOf)(xmlNode (*)));
 static PyObject *((*utf8)(PyObject *));
 static struct {char *s; void **p;} _etree_API[] = {
-  {"appendChild", &appendChild},
-  {"attributeValue", &attributeValue},
-  {"attributeValueFromNsName", &attributeValueFromNsName},
-  {"callLookupFallback", &callLookupFallback},
-  {"collectAttributes", &collectAttributes},
-  {"deepcopyNodeToDocument", &deepcopyNodeToDocument},
-  {"delAttribute", &delAttribute},
-  {"delAttributeFromNsName", &delAttributeFromNsName},
-  {"documentOrRaise", &documentOrRaise},
-  {"elementFactory", &elementFactory},
-  {"elementTreeFactory", &elementTreeFactory},
-  {"findChild", &findChild},
-  {"findChildBackwards", &findChildBackwards},
-  {"findChildForwards", &findChildForwards},
-  {"findOrBuildNodeNs", &findOrBuildNodeNs},
-  {"getAttributeValue", &getAttributeValue},
-  {"getNsTag", &getNsTag},
-  {"initTagMatch", &initTagMatch},
-  {"iteratorStoreNext", &iteratorStoreNext},
-  {"iterattributes", &iterattributes},
-  {"lookupDefaultElementClass", &lookupDefaultElementClass},
-  {"lookupNamespaceElementClass", &lookupNamespaceElementClass},
-  {"makeElement", &makeElement},
-  {"namespacedName", &namespacedName},
-  {"namespacedNameFromNsName", &namespacedNameFromNsName},
-  {"newElementTree", &newElementTree},
-  {"nextElement", &nextElement},
-  {"previousElement", &previousElement},
-  {"pyunicode", &pyunicode},
-  {"rootNodeOrRaise", &rootNodeOrRaise},
-  {"setAttributeValue", &setAttributeValue},
-  {"setElementClassLookupFunction", &setElementClassLookupFunction},
-  {"setNodeText", &setNodeText},
-  {"setTailText", &setTailText},
-  {"tagMatches", &tagMatches},
-  {"tailOf", &tailOf},
-  {"textOf", &textOf},
-  {"utf8", &utf8},
+  {"appendChild", (void*)(&appendChild)},
+  {"attributeValue", (void*)(&attributeValue)},
+  {"attributeValueFromNsName", (void*)(&attributeValueFromNsName)},
+  {"callLookupFallback", (void*)(&callLookupFallback)},
+  {"collectAttributes", (void*)(&collectAttributes)},
+  {"deepcopyNodeToDocument", (void*)(&deepcopyNodeToDocument)},
+  {"delAttribute", (void*)(&delAttribute)},
+  {"delAttributeFromNsName", (void*)(&delAttributeFromNsName)},
+  {"documentOrRaise", (void*)(&documentOrRaise)},
+  {"elementFactory", (void*)(&elementFactory)},
+  {"elementTreeFactory", (void*)(&elementTreeFactory)},
+  {"findChild", (void*)(&findChild)},
+  {"findChildBackwards", (void*)(&findChildBackwards)},
+  {"findChildForwards", (void*)(&findChildForwards)},
+  {"findOrBuildNodeNs", (void*)(&findOrBuildNodeNs)},
+  {"getAttributeValue", (void*)(&getAttributeValue)},
+  {"getNsTag", (void*)(&getNsTag)},
+  {"initTagMatch", (void*)(&initTagMatch)},
+  {"iteratorStoreNext", (void*)(&iteratorStoreNext)},
+  {"iterattributes", (void*)(&iterattributes)},
+  {"lookupDefaultElementClass", (void*)(&lookupDefaultElementClass)},
+  {"lookupNamespaceElementClass", (void*)(&lookupNamespaceElementClass)},
+  {"makeElement", (void*)(&makeElement)},
+  {"namespacedName", (void*)(&namespacedName)},
+  {"namespacedNameFromNsName", (void*)(&namespacedNameFromNsName)},
+  {"newElementTree", (void*)(&newElementTree)},
+  {"nextElement", (void*)(&nextElement)},
+  {"previousElement", (void*)(&previousElement)},
+  {"pyunicode", (void*)(&pyunicode)},
+  {"rootNodeOrRaise", (void*)(&rootNodeOrRaise)},
+  {"setAttributeValue", (void*)(&setAttributeValue)},
+  {"setElementClassLookupFunction", (void*)(&setElementClassLookupFunction)},
+  {"setNodeText", (void*)(&setNodeText)},
+  {"setTailText", (void*)(&setTailText)},
+  {"tagMatches", (void*)(&tagMatches)},
+  {"tailOf", (void*)(&tailOf)},
+  {"textOf", (void*)(&textOf)},
+  {"utf8", (void*)(&utf8)},
   {0, 0}
 };
 
