@@ -52,14 +52,28 @@ class ETreeOnlyTestCase(HelperTestCase):
 
     def test_element_names(self):
         Element = self.etree.Element
-        
         el = Element('name')
         self.assertEquals(el.tag, 'name')
         el = Element('{}name')
         self.assertEquals(el.tag, 'name')
+
+    def test_element_name_empty(self):
+        Element = self.etree.Element
+        el = Element('name')
+        self.assertRaises(ValueError, Element, '{}')
+        self.assertRaises(ValueError, setattr, el, 'tag', '{}')
+
         self.assertRaises(ValueError, Element, '{test}')
         self.assertRaises(ValueError, setattr, el, 'tag', '{test}')
 
+    def test_element_name_colon(self):
+        Element = self.etree.Element
+        self.assertRaises(ValueError, Element, 'p:name')
+        self.assertRaises(ValueError, Element, '{test}p:name')
+
+        el = Element('name')
+        self.assertRaises(ValueError, setattr, el, 'tag', 'p:name')
+
     def test_attribute_set(self):
         # ElementTree accepts arbitrary attribute values
         # lxml.etree allows only strings
@@ -173,6 +187,26 @@ class ETreeOnlyTestCase(HelperTestCase):
             '<a><b><c/></b></a>',
             tostring(tree))
 
+    def test_parse_remove_pis(self):
+        parse = self.etree.parse
+        tostring = self.etree.tostring
+        XMLParser = self.etree.XMLParser
+
+        xml = '<?test?><a><?A?><b><?B?><c/></b><?C?></a><?tail?>'
+
+        f = StringIO(xml)
+        tree = parse(f)
+        self.assertEquals(
+            xml,
+            tostring(tree))
+
+        f = StringIO(xml)
+        parser = XMLParser(remove_pis=True)
+        tree = parse(f, parser)
+        self.assertEquals(
+            '<a><b><c/></b></a>',
+            tostring(tree))
+
     def test_parse_parser_type_error(self):
         # ET raises IOError only
         parse = self.etree.parse
@@ -1229,6 +1263,39 @@ class ETreeOnlyTestCase(HelperTestCase):
             [d, f],
             list(a.getiterator('{b}*')))
 
+    def test_getiterator_filter_element(self):
+        Element = self.etree.Element
+        Comment = self.etree.Comment
+        PI = self.etree.PI
+        SubElement = self.etree.SubElement
+
+        a = Element('a')
+        b = SubElement(a, 'b')
+        a.append(Comment("test"))
+        a.append(PI("pi", "content"))
+        c = SubElement(a, 'c')
+
+        self.assertEquals(
+            [a, b, c],
+            list(a.getiterator(Element)))
+
+    def test_getiterator_filter_all_comment_pi(self):
+        # ElementTree iterates over everything here
+        Element = self.etree.Element
+        Comment = self.etree.Comment
+        PI = self.etree.PI
+        SubElement = self.etree.SubElement
+
+        a = Element('a')
+        b = SubElement(a, 'b')
+        a.append(Comment("test"))
+        a.append(PI("pi", "content"))
+        c = SubElement(a, 'c')
+
+        self.assertEquals(
+            [a, b, c],
+            list(a.getiterator('*')))
+
     def test_findall_ns(self):
         XML = self.etree.XML
         root = XML('<a xmlns:x="X" xmlns:y="Y"><x:b><c/></x:b><b/><c><x:b/><b/></c><b/></a>')
@@ -1378,6 +1445,18 @@ class ETreeOnlyTestCase(HelperTestCase):
         self.assertEquals(None, el.sourceline)
         self.assertEquals(None, child.sourceline)
 
+    def test_XML_base_url_docinfo(self):
+        etree = self.etree
+        root = etree.XML("<root/>", base_url="http://no/such/url")
+        docinfo = root.getroottree().docinfo
+        self.assertEquals(docinfo.URL, "http://no/such/url")
+
+    def test_HTML_base_url_docinfo(self):
+        etree = self.etree
+        root = etree.HTML("<html/>", base_url="http://no/such/url")
+        docinfo = root.getroottree().docinfo
+        self.assertEquals(docinfo.URL, "http://no/such/url")
+
     def test_docinfo_public(self):
         etree = self.etree
         xml_header = '<?xml version="1.0" encoding="ascii"?>'
